/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2020 Databases and Information Systems Group TU Dortmund           *
 * Visit us at                                                                  *
 *             http://dbis.cs.tu-dortmund.de/cms/en/home/                       *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *                                                                              *
 * Authors (alphabetical list):                                                 *
 *          Maximilian Berens   <maximilian.berens@tu-dortmund.de>              *
 *          Roland Kühn         <roland.kuehn@cs.tu-dortmund.de>                *
 *          Jan Mühlig          <jan.muehlig@tu-dortmund.de>                    *
 *------------------------------------------------------------------------------*
 */

#include <disk/lfu_strategy.h>
#include <exception/disk_exception.h>

using namespace beedb::disk;

std::size_t LFUStrategy::find_victim(const std::vector<FrameInformation> &frame_information)
{
    /**
     * Assignment (1): Implement the Least Frequently Used eviction strategy.
     *
     * This strategy evicts that frame from the frame buffer which
     * has no pins and is pinned the least times.
     *
     * This function returns the index of the frame that should be evicted.
     *
     * Hints for implementation:
     *  - The Parameter "frame_information" is a list of information about
     *    all frames and their current states.
     *  - You can get the information for the frame i by "frame_information[i]".
     *  - The information offers a method "is_pinned()" that returns true, when
     *    the frame is pinned at the moment, false otherwise.
     *  - The information offers a method "count_all_pins()" that returns the number
     *    of all pins (not only active pins). The number is will be reset on eviction.
     *
     * Procedure:
     *  - Scan the frame_information for frames that are not pinned at the moment.
     *  - Select the frame with the lowest number of all pins.
     *  - Return the index of that frame.
     */

    std::size_t evict_index = 0u;


    return evict_index;
}