/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2020 Databases and Information Systems Group TU Dortmund           *
 * Visit us at                                                                  *
 *             http://dbis.cs.tu-dortmund.de/cms/en/home/                       *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *                                                                              *
 * Authors (alphabetical list):                                                 *
 *          Maximilian Berens   <maximilian.berens@tu-dortmund.de>              *
 *          Roland Kühn         <roland.kuehn@cs.tu-dortmund.de>                *
 *          Jan Mühlig          <jan.muehlig@tu-dortmund.de>                    *
 *------------------------------------------------------------------------------*
 */

#include <disk/lru_k_strategy.h>
#include <exception/disk_exception.h>

using namespace beedb::disk;

std::size_t LRUKStrategy::find_victim(const std::vector<FrameInformation> &frame_information)
{
    /**
     * Assignment (1): Implement the Least Recently Used eviction strategy with regard
     *                 to the last "k" pins.
     *
     * The normal "LRU" strategy is a special case of this strategy,
     * using k = 1: LRU == LRU-1
     *
     * The LRU-K strategy takes not just the last reference of a frame into account,
     * but the k-th last reference. For more details, take a look into the original
     * paper for LRU-K: "The LRU-K page replacement algorithm for database disk buffering".
     *
     * This function returns the index of the frame that should be evicted.
     *
     * Hints for implementation:
     *  - The Parameter "frame_information" is a list of information about
     *    all frames and their current states.
     *  - The method "this->k()" returns the parameter "k" for this strategy.
     *  - You can get the information for the frame i by "frame_information[i]".
     *  - The information offers a method "is_pinned()" that returns true, when
     *    the frame is pinned at the moment, false otherwise.
     *  - "frame_information[i].pin_timestamp(j)" returns the e j-th timestamp a
     *    query pinned the frame "i".
     *
     * Procedure:
     *  - Scan the frame_information for frames that are not pinned at the moment
     *    and have a (1) overall number of pins lesser than k or (2) greater equal number of pins.
     *  - If (1) is not empty: Select the frame with the lowest timestamp of the last pin.
     *  - Otherwise select the frame with the lowest timestamp of the last k pins from (2).
     *  - Return the index to that frame.
     */

    std::size_t evict_index = 0u;


    return evict_index;
}