/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2020 Databases and Information Systems Group TU Dortmund           *
 * Visit us at                                                                  *
 *             http://dbis.cs.tu-dortmund.de/cms/en/home/                       *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *                                                                              *
 * Authors (alphabetical list):                                                 *
 *          Maximilian Berens   <maximilian.berens@tu-dortmund.de>              *
 *          Roland Kühn         <roland.kuehn@cs.tu-dortmund.de>                *
 *          Jan Mühlig          <jan.muehlig@tu-dortmund.de>                    *
 *------------------------------------------------------------------------------*
 */

#include <disk/lru_strategy.h>
#include <exception/disk_exception.h>

using namespace beedb::disk;

std::size_t LRUStrategy::find_victim(const std::vector<FrameInformation> &frame_information)
{
    /**
     * Assignment (1): Implement the Least Recently Used eviction strategy.
     *
     * This strategy evicts that frame from the frame buffer which
     * is not pinned at the moment and is pinned lately.
     *
     * This function returns the index of the frame that should be evicted.
     *
     * Hints for implementation:
     *  - The Parameter "frame_information" is a list of information about
     *    all frames and their current states.
     *  - You can get the information for the frame i by "frame_information[i]".
     *  - The information offers a method "is_pinned()" that returns true, when
     *    the frame is pinned at the moment, false otherwise.
     *  - "frame_information[i].last_pin_timestamp()" returns the timestamp of the last time a
     *    query pinned the frame "i".
     *
     * Procedure:
     *  - Scan the frame_information for frames that are not pinned at the moment.
     *  - Select the frame with the lowest timestamp of the last pin.
     *  - Return the index of that frame.
     */

    std::size_t evict_index = 0u;


    return evict_index;
}