/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2020 Databases and Information Systems Group TU Dortmund           *
 * Visit us at                                                                  *
 *             http://dbis.cs.tu-dortmund.de/cms/en/home/                       *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *                                                                              *
 * Authors (alphabetical list):                                                 *
 *          Maximilian Berens   <maximilian.berens@tu-dortmund.de>              *
 *          Roland Kühn         <roland.kuehn@cs.tu-dortmund.de>                *
 *          Jan Mühlig          <jan.muehlig@tu-dortmund.de>                    *
 *------------------------------------------------------------------------------*
 */

#pragma once
#include "page.h"
#include <cstdint>
#include <fstream>
#include <string>

namespace beedb::disk
{
/**
 * The StorageManager grants access to the data written to the disk.
 */
class StorageManager
{
  public:
    StorageManager(const std::string &file_name);
    ~StorageManager();

    /**
     * Copy a page from disk to memory.
     *
     * @param page_id Id of the page.
     * @return Data, now copied to memory.
     */
    Page read(const Page::page_id page_id);

    /**
     * Write the page from memory to disk.
     *
     * @param page Page to be written.
     */
    void write(Page &page);

    /**
     * Allocates a new page in the disk file and extends the
     * file by the new allocated page.
     *
     * @return Id of the new allocated page.
     */
    Page::page_id allocate();

    /**
     * @return Number of pages stored in the disk file.
     */
    inline std::size_t count_pages() const
    {
        return _count_pages;
    }

  private:
    std::size_t _count_pages = 0u;
    std::fstream _storage_file;
};
} // namespace beedb::disk