/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2020 Databases and Information Systems Group TU Dortmund           *
 * Visit us at                                                                  *
 *             http://dbis.cs.tu-dortmund.de/cms/en/home/                       *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *                                                                              *
 * Authors (alphabetical list):                                                 *
 *          Maximilian Berens   <maximilian.berens@tu-dortmund.de>              *
 *          Roland Kühn         <roland.kuehn@cs.tu-dortmund.de>                *
 *          Jan Mühlig          <jan.muehlig@tu-dortmund.de>                    *
 *------------------------------------------------------------------------------*
 */

#pragma once
#include "unary_operator.h"
#include <cstdint>
#include <table/table_disk_manager.h>
#include <table/value.h>
#include <utility>
#include <vector>

namespace beedb::execution
{
class UpdateOperator : public UnaryOperator
{
  public:
    UpdateOperator(table::TableDiskManager &table_disk_manager,
                   std::vector<std::pair<table::Schema::ColumnIndexType, table::Value>> &values)
        : _table_disk_manager(table_disk_manager), _new_column_values(std::move(values))
    {
    }
    virtual ~UpdateOperator() = default;

    virtual void open();
    virtual util::optional<table::Tuple> next();
    virtual void close();

    virtual const table::Schema &schema() const
    {
        return _schema;
    }

  private:
    table::Schema _schema;
    table::TableDiskManager _table_disk_manager;
    std::vector<std::pair<table::Schema::ColumnIndexType, table::Value>> _new_column_values;
};
} // namespace beedb::execution