/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2020 Databases and Information Systems Group TU Dortmund           *
 * Visit us at                                                                  *
 *             http://dbis.cs.tu-dortmund.de/cms/en/home/                       *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *                                                                              *
 * Authors (alphabetical list):                                                 *
 *          Maximilian Berens   <maximilian.berens@tu-dortmund.de>              *
 *          Roland Kühn         <roland.kuehn@cs.tu-dortmund.de>                *
 *          Jan Mühlig          <jan.muehlig@tu-dortmund.de>                    *
 *------------------------------------------------------------------------------*
 */

#pragma once

#include "b_plus_tree.h"
#include <index/index_interface.h>
#include <index/non_unique_index_interface.h>
#include <index/range_index_interface.h>

namespace beedb::index::bplustree
{
/**
 * B+-Tree implementation for non-unique (key,value) pairs.
 * Can store multiple values for each key.
 * Supports range queries.
 */
class NonUniqueBPlusTreeIndex : public IndexInterface, public NonUniqueIndexInterface, public RangeIndexInterface
{
  public:
    NonUniqueBPlusTreeIndex(const std::string &name) : IndexInterface(name)
    {
    }
    virtual ~NonUniqueBPlusTreeIndex() = default;

    virtual bool supports_range() const
    {
        return true;
    }
    virtual bool is_unique() const
    {
        return false;
    }

    virtual void put(const std::int64_t key, disk::Page::page_id page_pointer)
    {
        _tree.put(key, page_pointer);
    }

    virtual std::optional<std::set<disk::Page::page_id>> get(const std::int64_t key) const
    {
        return _tree.get(key);
    }

    virtual std::optional<std::set<disk::Page::page_id>> get(const std::int64_t key_from, const std::int64_t key_to)
    {
        return _tree.get(key_from, key_to);
    }

  private:
    BPlusTree<std::int64_t, disk::Page::page_id, false> _tree;
};
} // namespace beedb::index::bplustree