/*------------------------------------------------------------------------------*
 * Architecture & Implementation of DBMS                                        *
 *------------------------------------------------------------------------------*
 * Copyright 2020 Databases and Information Systems Group TU Dortmund           *
 * Visit us at                                                                  *
 *             http://dbis.cs.tu-dortmund.de/cms/en/home/                       *
 *                                                                              *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS      *
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,  *
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL      *
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR         *
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR        *
 * OTHER DEALINGS IN THE SOFTWARE.                                              *
 *                                                                              *
 * Authors (alphabetical list):                                                 *
 *          Maximilian Berens   <maximilian.berens@tu-dortmund.de>              *
 *          Roland Kühn         <roland.kuehn@cs.tu-dortmund.de>                *
 *          Jan Mühlig          <jan.muehlig@tu-dortmund.de>                    *
 *------------------------------------------------------------------------------*
 */

#pragma once
#include "custom_command_interface.h"

namespace beedb::io::command
{
class ShowCommand final : public CustomCommandInterface
{
  public:
    virtual ~ShowCommand() = default;
    virtual std::optional<Query> execute(const std::string &parameters) override;
    virtual std::string help_str() override
    {
        return std::string(_help_str);
    };

  private:
    static constexpr auto _help_str = "Syntax> :show [tables,indices,columns]{1}";
};

class ExplainCommand final : public CustomCommandInterface
{
  public:
    virtual ~ExplainCommand() = default;
    virtual std::optional<Query> execute(const std::string &input) override;
    virtual std::string help_str() override
    {
        return std::string(_help_str);
    };

  private:
    static constexpr auto _help_str = "Syntax> :explain [plan,graph]? <SQL-Statement>";
};

class SetCommand final : public CustomCommandInterface
{
  public:
    SetCommand(Config &config) : _config(config)
    {
    }
    virtual ~SetCommand() = default;
    virtual std::optional<Query> execute(const std::string &input) override;

    virtual std::string help_str() override
    {
        return std::string(_help_str);
    };

  private:
    static constexpr auto _help_str = "Syntax> :set <Argument Name> <Numerical Value>";

  private:
    Config &_config;
};

class GetCommand final : public CustomCommandInterface
{
  public:
    GetCommand(Config &config) : _config(config)
    {
    }
    virtual ~GetCommand() = default;
    virtual std::optional<Query> execute(const std::string &input) override;

    virtual std::string help_str() override
    {
        return std::string(_help_str);
    };

  private:
    static constexpr auto _help_str = "Syntax> :get [<Argument Name>]?";

  private:
    Config &_config;
};
} // namespace beedb::io::command